#!/usr/bin/python3

import subprocess
import json
from sre_compile import isstring
import logging
import os
from configparser import DuplicateOptionError, ConfigParser
from .RMOsInfo import RMOsInfo

RMSUCCESS = 0
RMFAILURE = -1

SEC_DESKTOP = 'Desktop Entry'

def get_sys_call(cmd):
    ret = ''
    if not isstring(cmd):
        logging.error("[RMCOM]: cmd is not string")

    try:
        with os.popen(cmd) as fd:
            ret = fd.read()
        fd.close()
        logging.info("[RMCOM]: sys call (%s) return[%s]", cmd, ret.strip("\n"))
    except Exception as e:
        logging.error("[RMCOM]: [%s] failed as [%s].", cmd, e)

    return ret

def str_to_list(str, symbol):
    if not isstring(str):
        logging.warning("[RMCOM]: str is not string")
        return None
    if symbol not in ["\n", "\t", "\'", "\"", " "]:
        logging.warning("[RMCOM]: symbol[%s] is not included.", symbol)
        return None

    strlist = []
    strwhile = str
    strindex = strwhile.find(symbol)
    while strindex!= -1:
        strtmp = strwhile[:strindex]
        strwhile = strwhile[strindex+1:]
        strlist.append(strtmp)
        strindex = strwhile.find(symbol)

    return strlist

def get_desktop_key(filePath, key):
    value = None
    conf = ConfigParser()
    if not os.path.exists(filePath):
        logging.error("[RMCOM]: %s not exist", filePath)
        return value
    try:
        conf.read(filePath, encoding="utf-8")
    except DuplicateOptionError:
        try:
            with open(filePath, 'r') as fd:
                infoList = fd.readlines()
            for ele in infoList:
                retList = ele.strip("\n").split("=")
                if len(retList) == 2 and retList[0] == key:
                    value = retList[1]
                    return value
        except Exception as e:
            logging.error("[RMCOM]: err[%s]", e)
            return None
    except Exception as e:
        logging.error("[RMCOM]: read %s failed err[%s]", filePath, e)
        return None

    if not conf.has_option(SEC_DESKTOP, key):
        logging.info("[RMCOM]: There is not key[%s].", key)
        return None

    try:
        value = conf.get(SEC_DESKTOP, key)
        logging.info("[RMCOM]: Get key[%s] value[%s]", key, value)
    except Exception as e:
        logging.error("[RMCOM]: err[%s]", e)

    return value

def set_desktop_key(filePath, key, value):
    try:
        infoList = []
        with open(filePath, 'r') as fd_r:
            infoList = fd_r.readlines()
        for ele in infoList:
            retList = ele.strip("\n").split("=")
            if len(retList) == 2 and retList[0] == key:
                rowNew = key + "=" + value + "\n"
                infoList[infoList.index(ele)] = ele.replace(ele, rowNew)
                logging.info("[RMCOM]: change [%s]", rowNew)
                break
        with open(filePath, 'w+') as fd_w:
            fd_w.writelines(infoList)
    except Exception as e:
        logging.error("[RMCOM]: set key[%s] err[%s]", key, e)


# 获取剩余空间信息
def get_path_space_info(mountPoint):
    sv = os.statvfs(mountPoint)
    freeSpace = sv.f_bavail * sv.f_bsize
    return freeSpace

# [
#     {"name":"nvme0n1p5","fsavail":"40.1G","mountpoint":"/data"}
# ]
# 与根分区在同一硬盘上
# 记录挂载点 -- 剩余空间
def get_disk_space_info():
    cmd = "lsblk --json --output name,type,mountpoint"
    ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if ret.returncode != 0:
        logging.error("[RMCOM]: [%s] err [%s].", cmd, ret.stderr.decode().strip("\n"))
        return None

    try:
        mainIsFind = False
        infoJson_d = ret.stdout.decode()
        infoJson_tmp = infoJson_d.replace('\\x','%')
        infoJson = json.loads(infoJson_tmp)
        for eleInfo in infoJson["blockdevices"]:
            if eleInfo["type"] != "disk":
                continue
            # if "children" not in eleInfo and "mountpoint" in eleInfo and eleInfo["mountpoint"] == "/":
                # mainIsFind = True
            if "children" not in eleInfo:
                continue

            for ele in eleInfo["children"]:
                if ele["mountpoint"] == "/":
                    mainIsFind = True
                    break

            if mainIsFind:
                spaceInfoList = []
                for ele in eleInfo["children"]:
                    spaceInfo = {}
                    spaceInfo["mountpoint"] = ele["mountpoint"]
                    spaceInfo["name"] = ele["name"]
                    if ele["mountpoint"] != None and ele["mountpoint"].startswith("/"):
                        spaceInfo["fsavail"] = get_path_space_info(ele["mountpoint"])
                        spaceInfoList.append(spaceInfo)

                return spaceInfoList
    except Exception as e:
        logging.error("[RMCOM]: get disk space info err[%s].", e)

    logging.error("[RMCOM]: Not find root space info[%s].", infoJson_d)
    return None

# 镜像下载的路径
def get_download_path(isoSize):
    path = None
    spaceInfoList = get_disk_space_info()
    if spaceInfoList == None:
        logging.error("[RMCOM] get disk info is none.")
        return path
    try:
        for spaceInfo in spaceInfoList:
            if spaceInfo["fsavail"] < isoSize:
                continue
            if spaceInfo["mountpoint"] == "/" and spaceInfo["fsavail"] > 12 * RMCom.SPACE_G + isoSize:
                logging.info("[RMCOM]: space info [%s].", spaceInfo)
                path = spaceInfo["mountpoint"] + ".tmpiso/"
                break
            elif spaceInfo["mountpoint"] != "/" and spaceInfo["fsavail"] > isoSize:
                logging.info("[RMCOM]: space info [%s].", spaceInfo)
                path = spaceInfo["mountpoint"] + "/.tmpiso/"
                break
    except Exception as e:
        logging.error("[RMCOM] get download path err[%s].", e)

    logging.info("[RMCOM]: download path [%s].", path)
    return path


class RMCom():
    VMINIT = 0
    VMUPACT2 = 1
    VMUP = 2
    VMUPSUCCESS = 3
    VMUPFAILED = 4
    VMBACKACT2 = 5
    VMBACK = 6 
    VMBACKSUCCESS = 7
    VMBACKFAILED = 8

    intToStr = {
        VMINIT : "none",
        VMUPACT2 : "upact2",
        VMUP : "upover",
        VMUPSUCCESS : "upsuccess",
        VMUPFAILED : "upfailed",
        VMBACKACT2 : "backact2",
        VMBACK : "backover",
        VMBACKSUCCESS : "backsuccess",
        VMBACKFAILED : "backfailed",
    }

    strToInt = {
        "none" : VMINIT,
        "upact2" : VMUPACT2,
        "upover" : VMUP,
        "upsuccess" : VMUPSUCCESS,
        "upfailed" : VMUPFAILED,
        "backact2" : VMBACKACT2,
        "backover" : VMBACK,
        "backsuccess" : VMBACKSUCCESS,
        "backfailed" : VMBACKFAILED,
    }
    
    LANGZH_LIST = ["zh_CN", "zh_CN:" "zh_CN:en", "zh_CN:zh", "zh_CN.UTF-8"]
    LANGEN_LIST = ["en", "en_US", "en_US.UTF-8"]

    SPACE_G = 1024*1024*1024

    # state sec/key
    SEC_HW_INFO     = 'hwInfo'
    KEY_ARCH        = 'arch'

    SEC_HOME_INFO   = 'homeInfo'
    KEY_USERS       = 'userlist'

    SEC_TIME        = 'time'
    KEY_UPTIME      = 'uptime'

    SEC_STATE       = 'state'
    KEY_ACTIVATED   = 'activated'
    KEY_VM          = 'versionmanager'
    KEY_STEP        = 'step'
    KEY_REASEON     = 'reason'
    KEY_USED        = 'usedcount'
    KEY_RUNNINGTIME = 'runningcount'
    KEY_CLEAR       = 'cleared'
    KEY_CLEARSTAGE  = 'clearstage'

    SEC_OS_INFO     = 'osInfo'
    KEY_OLD_KERNEL  = 'oldkerversion'
    KEY_RUN_KERNEL  = 'runkerversion'
    KEY_NEW_KERNEL  = 'newkerversion'
    KEY_OLD_FS_NAME = 'oldfsname'
    KEY_RUN_FS_NAME = 'runfsname'

    ARCHLIST                = [['arm','arm64','aarch64'],['amd','amd64','x86','x86_64','i386'],['armhf'],['loongarch64'],['mips64el','mips64'],['sw64']]

    KYINFO_NAME             = ".kyinfo"
    LICENSE_NAME            = "LICENSE"
    CACHE_DIR               = "/var/cache/RevisionsManager/"
    ETC_DIR                 = "/etc/RevisionsManager/"
    RM_STATE_FIEL           = "RevisionsManager.state"
    RM_STATE_PATH           = ETC_DIR + RM_STATE_FIEL
    CONF_FILE               = "RevisionsManager.conf"
    CONF_PATH               = ETC_DIR + CONF_FILE
    MD5_FILE                = "RevisionsManager.md5sum"
    MD5_PATH                = ETC_DIR + MD5_FILE

    NEWFS_DIR               = "/fs.new/"
    GRUB_FILE_PATH          = "/etc/grub.d/10_linux"

    APPDESKTOP_DIR          = "/usr/share/applications/"
    LOG_DIR                 = "/var/log/RevisionsManager/"
    SCRIPTS_DIR             = "/usr/share/revisions-manager/RevisionsManager/scripts/"
    SCRIPT_BEFORE           = 'upbefore/main'

    SCRIPT_POSTISNTALL      = "postinstall"
    # 安全相关 kysec pam
    SCRIPT_SECURITY         = "security"
    CMD_AUDIT               = "open_audit.sh"

    COMPAT_INFO_FILE        = "package-desktop.info" # 兼容需要
    KEY_ZH_CN_NAME          = 'Name[zh_CN]'
    KEY_NAME                = 'Name'
    KEY_ICON_PATH           = 'Icon'

    ORGFS_DIR               = "/fs.old"
    def __init__(self):
        self.final_fs_name()

    def final_fs_name(self):
        postfix = ''
        osInfoHD = RMOsInfo()
        osName = osInfoHD.get_pretty_name()
        if "ubuntu" in osName or "Ubuntu" in osName:
            tmpList = osName.split()
            if len(tmpList) >= 2:
                postfix ='-u' + tmpList[1].split(".")[0]
            else:
                postfix = '-u'
        elif "openKylin" in osName:
            tmpList = osName.split()
            if len(tmpList) >= 2:
                postfix ='-ok' + tmpList[1].split(".")[0]
            else:
                postfix = '-ok'
        elif "Kylin V10 SP1" in osName:
            postfix = '-v10sp1'
        elif "Kylin V10" in osName:
            postfix = '-v10'
        elif "Kylin V4" in osName:
            postfix = '-v4'
        elif "NeoKylin V7" in osName:
            postfix = '-v7'
        else:
            postfix = '-v10'

        self.ORGFS_DIR  = "/fs.old" + postfix
        logging.info("[RMCOM]: get pretty name[%s] oldfs[%s].", osName, self.ORGFS_DIR)

    def lang_in_list(self, langList):
        if not "LANG" in os.environ and not "LANGUAGE" in os.environ:
            logging.warning("[RMCOM]: no lang or language in environ.")
            return False
        elif "LANG" in os.environ and os.environ["LANG"] in langList:
            logging.info("[RMCOM]: LANG[%s].", os.environ["LANG"])
            return True
        elif "LANGUAGE" in os.environ and os.environ["LANGUAGE"] in langList:
            logging.info("[RMCOM]: LANGUAGE[%s].", os.environ["LANGUAGE"])
            return True
        else:
            logging.info("[RMCOM]: LANG is not in [%s].", langList)
            return False

    # 获取用户账户
    def get_usrs(self):
        usrList = []
        cmd = "lslogins -u --output UID,USER"
        retS = get_sys_call(cmd)
        retList = retS.split("\n")
        for everyLine in retList:
            eleList = everyLine.split()
            if len(eleList) != 2:
                logging.warning("[RMCOM] Get users %s.", eleList)
            if len(eleList) < 2:
                continue
            if eleList[0] == "UID":
                continue
            try:
                uid = int(eleList[0])
                if uid >= 1000:
                    usrList.append(eleList[1])
            except Exception as e:
                logging.error("[RMCOM] Get users err [%s].", e)

        logging.info("[RMCOM] Get users %s.", usrList)
        return usrList

if __name__ == "__main__":

    comClass = RMCom()

    # test-a
    # path = "/usr/share/applications/cn.lanxin.desktop"
    # value = get_desktop_key(path, "Icon")
    # print(value)
    # set_desktop_key(path, "Icon", "aaaa")

    # test-b -ok
    # get_disk_space_info()
    # get_download_path()

    # test c -ok
    retlist = comClass.get_usrs()
    print(retlist)



    pass
