#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import logging
import os
from gettext import gettext as _
from enum import IntEnum
from time import sleep
import shutil
import subprocess
import dbus
import requests
import copy
import json
import threading
from datetime import datetime, timedelta

from .RMRemote import RMRemote
from .RMConf import RMConf
from .RMGrub import RMGrub
from .RMMoveEtc import RMMoveEtc
from .RMApp import RMApp
from .RMCom import RMCom, RMSUCCESS, RMFAILURE, get_sys_call, str_to_list
from .RMScript import RMScriptDir
from .RMDepends import RMDepends
from .RMOsInfo import RMOsInfo
from .RMClear import RMClear
from .RMDaqEx import RMDaqEx

# 升级前检测项
UCHandlerIterms=(   _("This cpu is not supported"),
                    _("Please plug in or charge over 50%."),
                    _("Please prepare more than 12G space in the root partition"),
                    _("Please use right archive ISO"))

UCIterms=(  _("Check CPU whether is supported"),
            _("Check power whether is enough"),
            _("Check root free space"),
            _("Check iso right"))

class UCItermsEnum(IntEnum):
    UCCPU = 0
    UCPOWER = 1
    UCRootFree = 2
    UCIsoRight = 3
    UCItermsMax = 4

# 升级项
UPHandlerIterms=(_("Please run \"sudo update-initramfs -u\", and try again."),
                _("Please choose right ISO, and try again"),
                "",
                _("Please reboot, and try again"),
                _("Please reboot, and try again"),
                _("Please reboot, and try again"))
UPIterms=(  _("Create orginal system initrd"),
            _("Create new system"),
            _("Create new applications"),
            _("Move etc"),
            _("Create new system initrd"),
            _("Create new system grub"))
class UPItermsEnum(IntEnum):
    UPCreateOrgFsInitrd = 0
    UPCreateNewFs = 1
    UPCreateNewDeb = 2
    UPMoveEtc = 3
    UPCreateNewFsInitrd = 4
    UPCreateNewGrub = 5
    UPItermsMax = 6


# 升级后处理
UPPostIterms=(  _('update grub'),
                _('os config'),
                _('move security config'),
                _('install third party packages'),
                _('remove temp files'),
                _('update initrd'),
                _("activate system"),
                _('update upgrade state')
)
class UPPostItermsEnum(IntEnum):
    POSTUP_GRUB = 0           # 更新grub
    POSTUP_OSCONFIG = 1       # 重启后配置
    POSTUP_SECCONFIG = 2      # 安全迁移
    POSTUP_THIRDDEB = 3       # 第三方应用安装
    POSTUP_REMOVE = 4         # 移除临时文件
    POSTUP_INITRD = 5         # 更新initrd
    POSTUP_ACTIVE = 6         # 系统激活
    POSTUP_NEWSTATE = 7       # 更新升级状态、时间
    POSTUP_MAX = 8



# 还原前检测项
FCHandlerIterms=(_("Please check whether the old os exists"),
                _("Please check whether the old initrd exists"))
FCIterms=(  _("Check old os"),
            _("Check org Initrd"))
class FCItermsEnum(IntEnum):
    FCCheckOldOs = 0
    FCCheckOrgInitrd = 1
    FCItermsMax = 2

# 还原项
FPHandlerIterms=(_("Please reboot, and try again"))
FPIterms=(_("Create Fallback Grub"),)
class FPItermsEnum(IntEnum):
    FPCreateGrub = 0
    FPItermsMax = 1

# iso挂载 ok
# isoPath 绝对路径
# return 挂载路径
def mount_iso(isoPath):
    if not isoPath.startswith("/"):
        logging.error("[mount_iso]: isoPath[%s] is not abs path.", isoPath)
        return None, "Err ISO path " + isoPath
    logging.info("[mount_iso]: mount start")

    rmNewFs="/media/RevisionsManagerNewFs"
    # 挂载路径是否存在
    flag = False # 是否需要挂载
    for i in range(0,10):
        rmNewFs_i = rmNewFs + str(i)
        logging.info("[mount_iso]: folder[%s] will mount", rmNewFs_i)
        if not os.path.exists(rmNewFs_i):
            os.makedirs(rmNewFs_i, mode=0o755)
            flag = True
            break
        else:
            cmd = "mount | grep " + rmNewFs_i
            mountinfo = get_sys_call(cmd)
            # TODO: 认为 mountinfo 只有一行信息
            if isoPath in mountinfo.split(" "):
                # 已经挂载
                logging.info("[mount_iso]: folder[%s] has been mounted", rmNewFs_i)
                return rmNewFs_i, ''

    if not flag:
        logging.error("[mount_iso]: folder %s(0-10) is in use , please release one of them",rmNewFs)
        return None, 'Mount folder in use'
    
    cmd = "mount \"" + isoPath + "\" " + rmNewFs_i
    logging.info("[mount_iso]: cmd:%s", cmd)
    cmdRet = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    mountedStr = "already mounted on"
    mountedStrLen = len(mountedStr)
    cmdRetEnd=len(cmdRet.stderr)
    errInfo = cmdRet.stderr.decode()
    # 挂载是失败处理
    if cmdRet.returncode != RMSUCCESS:
        indexTmp = errInfo.find(mountedStr)
        # iso已挂载
        if -1 != indexTmp:
            indexPath = indexTmp + mountedStrLen + 1
            rmNewFs_i = errInfo[indexPath : cmdRetEnd-2]
            logging.info("[mount_iso]: iso[%s] has been mounted on [%s]",isoPath, rmNewFs_i)
        else:
            logging.error("[mount_iso]: iso[%s] mount [%s] failed. Because [%s]", isoPath, rmNewFs_i, errInfo)
            return None, errInfo.strip("\n")

    logging.info("[mount_iso]: mount end")
    return rmNewFs_i, ''

# 解除挂载 ok
# mountPath 挂载目录
# rmMountPathFlag 是否删除挂载目录
def umount_iso(mountPath, rmMountPathFlag=True):
    cmd = "umount " + mountPath      
    try:
        os.system(cmd)
        if rmMountPathFlag:
            os.rmdir(mountPath)
    except Exception as e:
        logging.error("[umount_iso]: cmd[%s] failed as [%s]", cmd, e)

def start_pre_act():
    path = "/opt/apps/net.zhongfu.appma"
    if not os.path.exists(path):
        logging.info("[PRE-ACT] %s not exists.", path)
        return

    shPath = "/usr/share/revisions-manager/RevisionsManager/scripts/prepare_env.sh"
    if not os.path.exists(shPath):
        logging.info("[PRE-ACT] %s not exists.", shPath)
        return

    cmd = "bash " + shPath + " > /var/log/RevisionsManager/prepare.log"
    get_sys_call(cmd)

class RMVersionManager(RMCom):
    POST_UPPROCESS_LIST     = [60,65,70,85,90,92,95,98,100]  # 升级后处理进度划分 8
    UP_PROCESS_LIST         = [0, 3, 8, 45, 50, 55, 60]    # 升级进度划分 6
    KYSEC_CFG               = 'kysec.config'

    # 初始化 
    def __init__(self):
        start_pre_act()

        self.upCheckFlag    = RMFAILURE     # 升级前检测状态
        self.backCheckFlag  = RMFAILURE     # 还原前检测状态
        self.newVersion     = None          # 从新系统iso中获取
        self.oldVersion     = None          # 升级的系统记录
        self.kysecList      = []            # 关闭的安全选项
        self.postProcessLastIndex = 0
        self.postProcessRunningIndex = 0
        self.postProcessCount = 0
        self.up2ProcessCb = None
        self.upProcessCb = None
        self.upCheckProcessCb = None
        self.backProcessCb = None
        self.backCheckProcessCb = None
        self.daqFailedInfo = ''
        self.arch       = get_sys_call("uname -m").strip("\n")
        self.curVersion = get_sys_call("uname -r").strip("\n")
        self.osInfo     = RMOsInfo()
        if not os.path.exists(self.ETC_DIR):
            os.makedirs(self.ETC_DIR, mode=0o755)
        if not os.path.exists(self.CACHE_DIR):
            os.mkdir(self.CACHE_DIR, mode=0o755)
        self.rmConf = RMConf(self.CONF_PATH)  # 配置文件中开关
        self.appClass = RMApp()
        self.daqInfo = RMDaqEx()

        self.rmStateConf = RMConf(self.RM_STATE_PATH)
        self.rmStateConf.change_key_value(self.SEC_HW_INFO, self.KEY_ARCH, self.arch)

        self.final_fs_name()

        tmpThread = threading.Thread(target=self.appClass.find_repack_list)
        tmpThread.start()

        self.thread = threading.Thread(target=self._state_handler)
        self.thread.start()

        logging.info("[VM]: init finished")

    def _act_in_newos_remove_kylin_os_install(self):
        cmd = "apt purge -y kylin-os-installer"
        get_sys_call(cmd)
    
        path = "/usr/share/xsessions/start-kylin-os-installer.desktop"
        if os.path.exists(path):
            os.remove(path)

    def _act_in_newos(self):
        # Description=disable shortcuts
        # disableglobalshortcuts=true
        # disableglobalshortcutsrule=2
        # wmclass=revision.exec
        # wmclasscomplete=false
        # wmclassmatch=1
        SEC_GEN = "General"
        KEY_COUNT = "count"
        path = "/etc/xdg/ukui-kwinrulesrc"
        kwinConf = RMConf(path)
        try:
            countS = kwinConf.get_key(SEC_GEN, KEY_COUNT)
            if countS == None:
                count = 0
            else:
                count = int(countS)
            newSecCount = str(count + 1)
            kwinConf.change_key_value(newSecCount, "Description", "disable shortcuts")
            kwinConf.change_key_value(newSecCount, "disableglobalshortcuts", "true")
            kwinConf.change_key_value(newSecCount, "disableglobalshortcutsrule", "2")
            kwinConf.change_key_value(newSecCount, "wmclass", "revision.exec")
            kwinConf.change_key_value(newSecCount, "wmclasscomplete", "false")
            kwinConf.change_key_value(newSecCount, "wmclassmatch", "1")
            kwinConf.change_key_value(SEC_GEN, KEY_COUNT, newSecCount)
        except Exception as e:
            logging.error("[VM-UP]: change kwin info err[%s].", e)


    def _state_handler(self):
        logging.info("[VM]: state handle start")
        self.versionManagerState = self.rmStateConf.get_key(self.SEC_STATE, self.KEY_VM)
        vmStateInt = self.strToInt.get(self.versionManagerState)
        if vmStateInt == None:
            vmStateInt = self.VMINIT
            self.versionManagerState = self.intToStr.get(self.VMINIT)
            self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.versionManagerState)
        
        if vmStateInt >= self.VMUP and vmStateInt <= self.VMUPFAILED:
                self.oldVersion = self.rmStateConf.get_key(self.SEC_OS_INFO,self.KEY_OLD_KERNEL)

        forcePath = os.path.join(self.ETC_DIR, "forceback")
        if os.path.exists(forcePath):
            self.update_grub()
            logging.info("[VM-BACK]: post handle for force-back")
            os.remove(forcePath)
            self.daqInfo.set_step_and_reason("back-force-success", "")
            self.daqInfo.send_all_info()
            vmStateInt = self.VMBACKACT2

        path = self.ETC_DIR + "need-update-grub"
        if os.path.exists(path):
            logging.info("[VM-BACK]: post handle for manual back")
            self.update_grub()
            os.remove(path)
            vmStateInt = self.VMBACKACT2

        if vmStateInt == self.VMUPACT2:
            self._up_post_handle()
            self._act_in_newos_remove_kylin_os_install()
            return
        elif vmStateInt == self.VMBACKACT2:
            self._back_post_handle()
        elif vmStateInt == self.VMUPSUCCESS:
            self._act_in_newos()
            self._up_post_active()
        elif vmStateInt == self.VMUPFAILED:
            pass
        elif vmStateInt == self.VMUP:
            self._up_post_active()
            self._act_in_newos_remove_kylin_os_install()

        debHandler = RMDepends()
        if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
            debHandler.install_base_kylin()
        else:
            debHandler.install_base_openkylin()

        logging.info("[VM]: state handle end")
        return

    # 升级第二部分 进度显示函数
    def _update_process(self, index, hasProcess = False, programClass = None):
        stepName = ""
        stepProcess = 0 # 模块的实时进度
        process = 0

        if index < 0 or index >= UPPostItermsEnum.POSTUP_MAX:
            logging.error("[VM-UP][process]: index[%d] is err.", index)
        else:
            if self.postProcessRunningIndex > index:
                return
            self.postProcessRunningIndex = index

            stepName = UPPostIterms[index]
            try:
                # 外部程序 支持上报实时进度
                if hasProcess and programClass is not None:
                    tmpName, stepProcess = programClass.get_processState()
                    process = int(self.POST_UPPROCESS_LIST[index] + (self.POST_UPPROCESS_LIST[index + 1] - self.POST_UPPROCESS_LIST[index]) * stepProcess / 100)
                    self.postProcessLastIndex = index
                # 模拟进度
                else:
                    if self.postProcessLastIndex == index:
                        self.postProcessCount += 1
                        process = self.POST_UPPROCESS_LIST[index] + self.postProcessCount
                        if process >= self.POST_UPPROCESS_LIST[index+1]:
                            process = self.POST_UPPROCESS_LIST[index+1]
                    elif self.postProcessLastIndex > index:
                        return
                    else:
                        # 更新
                        self.postProcessLastIndex = index
                        process = self.POST_UPPROCESS_LIST[index]
                        self.postProcessCount = 0
            except Exception as e:
                logging.error("[VM-UP][process]: get process state err[%s].", e)

        logging.info("[VM-UP][process]: update name[%s] stepProcess[%d] process[%d].", stepName, stepProcess, process)
        self.show_up2_process(stepName, process)

        if stepProcess >= 100 or stepProcess < 0:
            return RMSUCCESS

        if process >= 100 or process < 0:
            return RMSUCCESS

        timerTmp = threading.Timer(1, function=self._update_process, args=[index, hasProcess, programClass])
        timerTmp.start()

    def _up_post_close_specify_security(self):
        tmpList = []
        kysecCfgPath = "/fs.old" + self.CACHE_DIR + self.KYSEC_CFG
        if not os.path.exists(kysecCfgPath):
            logging.info("[VM-UP]: [%s] not exists.", kysecCfgPath)
            return

        with open(kysecCfgPath, 'r') as fd:
            tmpList = fd.readlines()
        fd.close()

        # pwquality  userlock  exectl  devctl
        for ele in tmpList:
            option_sec = ele.strip("\n")
            if option_sec == "pwquality":
                cmd = "rm /usr/share/pam-configs/pwquality"
                get_sys_call(cmd)
                cmd = "pam-auth-update --package"
                get_sys_call(cmd)

            elif option_sec == "userlock":
                cmd = "cp /usr/share/ksc-defender-daemon/pam-config/unix /usr/share/pam-configs/"
                get_sys_call(cmd)
                cmd = "pam-auth-update --package"
                get_sys_call(cmd)

            elif option_sec == "exectl":
                cmd = "setstatus -f exectl off -p"
                get_sys_call(cmd)
            elif option_sec == "devctl":
                cmd = "setstatus -f devctl off -p"
                get_sys_call(cmd)
            else:
                logging.warning("[VM-UP]: not support this [%s]option.", ele)

    def _move_activate_file(self):
        ret = True
        kyinfoPathA = "/fs.old" + self.appClass.CACHE_DIR + self.KYINFO_NAME
        kyinfoPathB ="/fs.old" + self.appClass.CACHE_DIR + "download/" + self.KYINFO_NAME
        distPath = "/etc/"+self.KYINFO_NAME
        if os.path.exists(distPath) or os.path.islink(distPath):
            cmd = "rm -f " + distPath
            get_sys_call(cmd)
        if os.path.exists(kyinfoPathA):
            cmd = "cp " + kyinfoPathA + " /etc/"
            get_sys_call(cmd)
            logging.info("[VM-UP]: copy [%s] to [%s]", kyinfoPathA, distPath)
        elif os.path.exists(kyinfoPathB):
            cmd = "cp " + kyinfoPathB + " /etc/"
            get_sys_call(cmd)
            logging.info("[VM-UP]: copy [%s] to [%s]", kyinfoPathB, distPath)
        else:
            ret = False
            logging.warning("[VM-UP]: [%s] not exists.", self.KYINFO_NAME)

        licensePathA = "/fs.old" + self.appClass.CACHE_DIR + self.LICENSE_NAME
        licensePathB = "/fs.old" + self.appClass.CACHE_DIR + "download/" + self.LICENSE_NAME
        distPath = "/etc/" + self.LICENSE_NAME
        if os.path.exists(distPath) or os.path.islink(distPath):
            cmd = "rm -f " + distPath
            get_sys_call(cmd)
        if os.path.exists(licensePathA):
            cmd = "cp " + licensePathA + " /etc/"
            get_sys_call(cmd)
            logging.info("[VM-UP]: copy [%s] to [%s]", licensePathA, distPath)
        elif os.path.exists(licensePathB):
            cmd = "cp " + licensePathB + " /etc/"
            get_sys_call(cmd)
            logging.info("[VM-UP]: copy [%s] to [%s]", licensePathB, distPath)
        else:
            ret = False
            logging.warning("[VM-UP]: [%s] not exists.", self.LICENSE_NAME)

        return ret
    
    def _check_activate_file(self, index=0):
        distPathK = "/etc/"+self.KYINFO_NAME
        distPathL = "/etc/"+self.LICENSE_NAME
        if os.path.exists(distPathK) and os.path.exists(distPathL) and os.path.isfile(distPathK) and os.path.isfile(distPathL):
            logging.info("[VM-ACTIVE] File is already.")
            return True
        
        ret = self._move_activate_file()
        if ret:
            logging.info("[VM-ACTIVE] File is moved successfully.")
            return True

        if index < 10:
            sleep(30)
        else:
            sleep(600)

        rmRemote = RMRemote()
        rmRemote.get_license_from_sources_only(isOld=True)
        index = index + 1
        self._check_activate_file(index)

    
    def _active_act(self):
        logging.info("[VM-UP]:ACTIVE start.")

        self._check_activate_file()

        tmp = None
        try:
            bus = dbus.SystemBus()
            activation_proxy = bus.get_object("org.freedesktop.activation", "/org/freedesktop/activation")
            activation_bus_interface = dbus.Interface(activation_proxy, dbus_interface="org.freedesktop.activation.interface")
            func_status = activation_bus_interface.get_dbus_method("status")
            status = 0
            if func_status != None:
                status, tmp = func_status()
            else:
                logging.error("[VM-UP]:ACTIVE func status is none.")
            
            if status:
                logging.info("[VM-UP]: Is activated.")
                return

            func_date = activation_bus_interface.get_dbus_method("date")
            actDate = ''
            if func_status != None:
                actDate, tmp = func_date()
            else:
                logging.error("[VM-UP]:ACTIVE func date is none.")

            if len(actDate) > 0:
                logging.info("[VM-UP]: Activation date[%s] is over.", actDate)
                return

            func_get_encrypted_hardware_info = activation_bus_interface.get_dbus_method("get_encrypted_hardware_info")
            hw_info = ''
            if func_get_encrypted_hardware_info != None:
                hw_info, tmp = func_get_encrypted_hardware_info()
            else:
                logging.error("[VM-UP]:ACTIVE func get_encrypted_hardware_info is none.")        

            func_serial_number = activation_bus_interface.get_dbus_method("serial_number")
            serial_number = ''
            if func_serial_number != None:
                serial_number, tmp = func_serial_number()
            else:
                logging.error("[VM-UP] func serial_number is none.")

            oldhwS = ''
            path = "/fs.old" + self.CACHE_DIR + "hwinfo"
            if os.path.exists(path):
                with open(path, 'r') as fd:
                    oldhwS = fd.read()
                fd.close()
                if len(oldhwS) <= 2:
                    oldhwS = ''
                    logging.warning("[VM-UP]:ACTIVE can not get old hwinfo.")
            else:
                logging.warning("[VM-UP]:ACTIVE [%s] not exists.", path)

            if len(oldhwS) == 0:
                url="https://wx.kylinos.cn/api/get_activation.json?f=" + serial_number + "&y=" + hw_info
            else:
                url="https://wx.kylinos.cn/api/get_activation.json?f=" + serial_number + "&y=" + hw_info + "&oy=" + oldhwS.strip("\n")
            response = requests.post(url)
            if response.status_code != 200:
                logging.error("[VM-UP]:ACTIVE post[%s] err[%d].", url, response.status_code)
            else:
                infoJson = json.loads(response.content)
                key = ''
                logging.info("[VM-UP]:ACTIVE post[%s] return[%s].",url, infoJson)
                if "data" in infoJson and "KEY" in infoJson["data"]:
                    key = infoJson["data"]["KEY"]
                    if len(key) < 16:
                        logging.error("[VM-UP]:ACTIVE get KEY[%s] err.", key)

                    keyF = key.replace('-', '')
                    func_activate = activation_bus_interface.get_dbus_method("activate")
                    if func_activate != None:
                        ret = func_activate(keyF)
                        logging.error("[VM-UP]:ACTIVE ret[%d].",ret)
                    else:
                        logging.error("[VM-UP]:ACTIVE func activate is none.")
                else:
                    logging.error("[VM-UP]:ACTIVE no KEY info in JSON.")

        except Exception as e:
            logging.error("[VM-UP]:ACTIVE err[%s].", e) 

        self.activeTime = self.activeTime + 1
        if self.activeTime < 5:
            tmpTime = 120
        else:
            tmpTime = 7200
        
        logging.info("[VM-UP]:ACTIVE next waiting.") 
        timera = threading.Timer(tmpTime, function=self._active_act)
        timera.start()


    def _up_post_active(self):
        self.activeTime = 0
        self._active_act()

    def gather_old_log(self):
        pass

    # 升级第二部分处理 -- 第一次重启
    def _up_post_handle(self):
        logging.error("[VM-UP]: up post handle start.")

        self.show_up2_process(UPPostIterms[0], self.POST_UPPROCESS_LIST[0])

        try:
            # 1-更新grub
            timerc = threading.Timer(0.1, function=self._update_process,args=[UPPostItermsEnum.POSTUP_GRUB])
            timerc.start()
            ret = self.up_post_update_grub()
            if ret != RMSUCCESS:
                logging.error("[VM-UP] update grub failed.")
                self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.intToStr.get(self.VMUPFAILED))

            grubClass = RMGrub()
            shutil.copy2(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg-test-1"))
            postInstallClass = RMScriptDir(self.SCRIPT_POSTISNTALL, os.path.join(self.SCRIPTS_DIR, self.SCRIPT_POSTISNTALL))
            timera = threading.Timer(0.2, function=self._update_process,args=[UPPostItermsEnum.POSTUP_OSCONFIG, True, postInstallClass])
            timera.start()
            postInstallClass.run_dir()
            shutil.copy2(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg-test-2"))
            self.up_post_resume_grub()
            shutil.copy2(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg-test-3"))
            if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                securityClass = RMScriptDir(self.SCRIPT_SECURITY, os.path.join(self.SCRIPTS_DIR, self.SCRIPT_SECURITY))
                timerb = threading.Timer(0.2, function=self._update_process,args=[UPPostItermsEnum.POSTUP_SECCONFIG, True, securityClass])
                timerb.start()
                securityClass.run_dir()
            shutil.copy2(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg-test-4"))

            if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                self._up_post_close_specify_security()

            # 安装第三方软件包
            # 在ISO 自带包安装之后，防止第三方包安装失败导致都不可以安装
            timerc = threading.Timer(0.1, function=self._update_process,args=[UPPostItermsEnum.POSTUP_THIRDDEB])
            timerc.start()
            try:
                debHandler = RMDepends()
                if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                    debHandler.install_newos_kylin()
                    debHandler.install_newos_kylin_ex()
                    releaseId = self.osInfo.get_release_id()
                    if releaseId == '2303':
                        debHandler.install_newos_2303()
                else:
                    debHandler.install_newos_openkylin()

                self.appClass.up_post_handle()
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)

            # 2-清除多余的标志文件
            timerc = threading.Timer(0.1, function=self._update_process,args=[UPPostItermsEnum.POSTUP_REMOVE])
            timerc.start()
            try:
                os.system("rm -f /mv*Ok")
                if os.path.exists("/cdrom"):
                    logging.info("[VM-UP]: remove /cdrom.")
                    shutil.rmtree("/cdrom")
                initramfsPath = "/run/initramfs/initramfs.debug"
                if os.path.exists(initramfsPath):
                    cmd = "cp " + initramfsPath + " /var/log/RevisionsManager/"
                    os.system(cmd)

                # plymouth 改为默认, 在update-initramfs之前
                if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                    cmd = "update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth \
                            /usr/share/plymouth/themes/kylin-revision-manager-logo/kylin-revision-manager-logo.plymouth 50"
                else:
                    cmd = "update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth \
                            /usr/share/plymouth/themes/openkylin-revision-manager-logo/openkylin-revision-manager-logo.plymouth 50"
                get_sys_call(cmd)

                cmd = "update-alternatives --auto default.plymouth"
                get_sys_call(cmd)
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)

            # 4-更新initrd
            timerc = threading.Timer(0.1, function=self._update_process,args=[UPPostItermsEnum.POSTUP_INITRD])
            timerc.start()
            logging.info("[VM-UP]: update initrd.")
            try:
                os.system("update-initramfs -u -k " + self.curVersion)
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)

            # 6-写入升级成功的时间
            try:
                uptime = datetime.now()
                timeStr = datetime.strftime(uptime,"%Y-%m-%d %H:%M:%S")
                self.rmStateConf.change_key_value(self.SEC_TIME, self.KEY_UPTIME, timeStr)
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)

            # 7-重置状态
            try:
                rmClear = RMClear()
                rmClear.reset_clear_state()
                rmClear.set_switch(True)
                rmClear.reset_use_time()
                rmClear.reset_record_time()
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)


            # 8-启动兼容环境
            buildEnvPath = "/usr/bin/buildCompatibilityEnv.sh"
            if os.path.exists(buildEnvPath):
                cmd = "bash " + buildEnvPath
                get_sys_call(cmd)
            else:
                logging.error("[VM-UP]: build docker env shell not exists.")

            # 激活
            try:
                if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                    ret = self._move_activate_file()
                    if ret:
                        self._up_post_active()
                        self.show_up2_process(UPPostIterms[UPPostItermsEnum.POSTUP_ACTIVE], self.POST_UPPROCESS_LIST[UPPostItermsEnum.POSTUP_ACTIVE])
            except Exception as e:
                logging.error("[VM-UP]: post handle failed[%s].",e)

            # 写入升级成功状态
            self.show_up2_process(UPPostIterms[UPPostItermsEnum.POSTUP_NEWSTATE], self.POST_UPPROCESS_LIST[UPPostItermsEnum.POSTUP_NEWSTATE])
            self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.intToStr.get(self.VMUPSUCCESS))
            self.show_up2_process(UPPostIterms[UPPostItermsEnum.POSTUP_NEWSTATE], self.POST_UPPROCESS_LIST[UPPostItermsEnum.POSTUP_MAX])
            self.daqInfo.set_step_and_reason("up-act-success", "")
            self.daqInfo.send_all_info()
            logging.info("[VM-UP]: post handle finished, will reboot.")
            os.system("reboot")
            return

        except Exception as e:
            logging.error("[VM-UP]: post handle failed[%s].",e)
            self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.intToStr.get(self.VMUPFAILED))
            self.daqInfo.set_step_and_reason("up-act2-failed", UPPostIterms[self.postProcessLastIndex] + " failed: ")
            self.daqInfo.send_all_info()
        logging.info("[VM-UP]: post handle finished, will reboot.")
        os.system("reboot")

    def show_up2_process(self, stepName, process):
        if self.up2ProcessCb != None:
            self.up2ProcessCb(stepName, process)

    def show_up_process(self, stepName, state, process, suggestion):
        if self.upProcessCb != None:
            self.upProcessCb(stepName, state, process, suggestion)

    def show_up_check_process(self, stepName, state, process, suggestion):
        if self.upCheckProcessCb != None:
            self.upCheckProcessCb(stepName, state, process, suggestion)

    def show_back_process(self, stepName, state, process, suggestion):
        if self.backProcessCb != None:
            self.backProcessCb(stepName, state, process, suggestion)

    def show_back_check_process(self, stepName, state, process, suggestion):
        if self.backCheckProcessCb != None:
            self.backCheckProcessCb(stepName, state, process, suggestion)

    def reg_up2_process_func(self, func):
        self.up2ProcessCb = func
    def reg_up_process_func(self, func):
        self.upProcessCb = func
    def reg_upCheck_process_func(self, func):
        self.upCheckProcessCb = func
    def reg_back_process_func(self, func):
        self.backProcessCb = func
    def reg_backCheck_process_func(self, func):
        self.backCheckProcessCb = func


    ############# 升级 #############

    # 检查架构 arch x86
    def _check_iso_arch(self, path):
        efiPath = "/dists/"
        distPath = path + efiPath
        if not os.path.exists(distPath):
            logging.error("[VM-UP]: There is path [%s].", distPath)
            return RMFAILURE, "/dists not exists in ISO"
        cmd = "find " + distPath + " -name \"binary-*\""
        ret = get_sys_call(cmd)
        fileInfoList = str_to_list(ret, "\n")
        if len(fileInfoList) == 0:
            logging.error("[VM-UP]: Path [%s] no file.", distPath)
            return RMFAILURE, "/dists/binary- file not exists in ISO"

        archList = []
        for eleList in self.ARCHLIST:
            if self.arch in eleList:
                archList = copy.deepcopy(eleList)
                break
        if len(archList) == 0:
            logging.error("[VM-UP]: not find self arch[%s].", self.arch)
            return RMFAILURE, "Arch is " + self.arch + ", not supported"

        for fileInfo in fileInfoList:
            tmpInfoList = fileInfo.split("binary-")
            if len(tmpInfoList) < 2:
                logging.info("[VM-UP]: Info [%s] not right.", fileInfo)
                continue
            tmpInfo = tmpInfoList[1]
            for ele in archList:
                if ele in tmpInfo:
                    logging.info("[VM-UP]: info [%s]and[%s] is the same", tmpInfo , self.arch)
                    return RMSUCCESS, ''

        logging.info("[VM-UP]: arch [%s] and [%s] is not the same", tmpInfo , self.arch)
        return RMFAILURE, "Arch is " + self.arch + ", and iso arch info " + tmpInfo

    # 检测ISO正确
    def _check_iso_right(self, isoPath):
        ret = RMSUCCESS
        if not os.path.exists(isoPath):
            logging.error("[VM-UP]: iso[%s] not exist.", isoPath)
            self.daqFailedInfo = "ISO not exist, path is " + isoPath
            return RMFAILURE

        mountPath, errInfo = mount_iso(isoPath)
        if mountPath is None:
            logging.error("[VM-UP]: Mount [%s] failed.", isoPath)
            self.daqFailedInfo = "Mount ISO failed " + isoPath + ". As " + errInfo
            return RMFAILURE

        ret, errInfo = self._check_iso_arch(mountPath)
        if ret != RMSUCCESS:
            self.daqFailedInfo = "Check iso arch err. " + errInfo

        umount_iso(mountPath)
        logging.info("[VM-UP]: Check iso is ret[%d].", ret)
        return ret

    # 检测根分区空间 ok
    # TODO: 需要空间的阈值 12*SPACE_G 再确认
    def _check_root_free(self):
        sv = os.statvfs("/")
        freeSpace = sv.f_bavail * sv.f_bsize
        if freeSpace < 12*self.SPACE_G:
            logging.warning(_("root free space is not enough [12G]."))
            self.daqFailedInfo = "Root free space is " + str(freeSpace)
            return RMFAILURE
        return RMSUCCESS

    # 检测cpu 是否为不支持的型号
    def _check_cpu_support(self):
        notSupportList = ["Phytium,FT-1500", "Phytium,FT-1500/4", "Loongson-3A1500"]

        cmd = "lscpu | grep 型号名称"
        retS = get_sys_call(cmd)
        if len(retS) > 0:
            retList = retS.strip("\n").split(":")
            if len(retList) == 2 and retList[1].strip() in notSupportList:
                self.daqFailedInfo = "CPU is " + retList[1].strip()
                return RMFAILURE

        cmd = "lscpu | grep \"Model name:\""
        retS = get_sys_call(cmd)
        if len(retS) > 0:
            retList = retS.strip("\n").split(":")
            if len(retList) == 2 and retList[1].strip() in notSupportList:
                self.daqFailedInfo = "CPU is " + retList[1].strip()
                return RMFAILURE
        return RMSUCCESS

    # 检测电量
    def _check_power(self, threshold=50):
        try:
            bus = dbus.SystemBus()
            proxy = bus.get_object("org.freedesktop.UPower", "/org/freedesktop/UPower")
            bus_interface = dbus.Interface(proxy, dbus_interface="org.freedesktop.DBus.Properties")
            ret = False
            func_getProperity = bus_interface.get_dbus_method("Get")
            if func_getProperity != None:
                ret = func_getProperity("org.freedesktop.UPower", "OnBattery")
            if not ret:
                logging.info("[VM-UP] power is plug in [%d].", ret)
                return RMSUCCESS

            bus_interface = dbus.Interface(proxy, dbus_interface="org.freedesktop.UPower")
            func_getDisplayDevice = bus_interface.get_dbus_method("GetDisplayDevice")
            if func_getProperity != None:
                devicePath = func_getDisplayDevice()
            if len(devicePath) == 0:
                logging.error("[VM-UP] can not get device.")
                self.daqFailedInfo = 'Can not find upower display device'
                return RMFAILURE

            proxy = bus.get_object("org.freedesktop.UPower", devicePath)
            bus_interface = dbus.Interface(proxy, dbus_interface="org.freedesktop.DBus.Properties")
            func_getProperityEx = bus_interface.get_dbus_method("Get")
            if func_getProperityEx != None:
                value = func_getProperityEx("org.freedesktop.UPower.Device", "Percentage")
            if value > threshold or value == 0:
                logging.info("[VM-UP] power is [%d] over [%d].", value, threshold)
                return RMSUCCESS
        except Exception as e:
            logging.error("[VM-UP] check power err[%s].", e)

        self.daqFailedInfo = 'Power percentage is ' + str(value)
        return RMFAILURE
    
    def up_pre_check_every(self, index, func, args=''):
        PRE_CHECK_PROCESS_LIST = [0, 25, 50, 75, 100]
        count = 5
        sigSpeed = 4
        sug = ""
        self.daqFailedInfo = ''

        try:
            for i in range(count):
                self.show_up_check_process(UCIterms[index], 0, PRE_CHECK_PROCESS_LIST[index] + i * sigSpeed, "")
                sleep(0.5)

            if len(args) == 0:
                ret = func()
            else:
                ret = func(args)
            if ret != RMSUCCESS:
                sug = UCHandlerIterms[index]
                self.daqInfo.set_step_and_reason("up-check-failed", UCIterms[index] + " failed: " + self.daqFailedInfo)
            self.show_up_check_process(UCIterms[index], ret, PRE_CHECK_PROCESS_LIST[index+1], sug)
            self.upCheckFlag += ret
        except Exception as e:
            logging.error("[VM-UP] up pre check every err[%s].", e)
            self.show_up_check_process(UCIterms[index], -1, PRE_CHECK_PROCESS_LIST[index+1], "")
            self.daqInfo.set_step_and_reason("up-check-failed", UCIterms[index] + " failed: " + self.daqFailedInfo)

    
    def up_pre_check(self, isoPath):
        rmRemote = RMRemote()
        tmpThread = threading.Thread(target=rmRemote.get_license_from_sources_only)
        tmpThread.start()

        self.upCheckFlag = RMSUCCESS
        self.up_pre_check_every(UCItermsEnum.UCCPU, self._check_cpu_support)

        self.up_pre_check_every(UCItermsEnum.UCPOWER, self._check_power)

        self.up_pre_check_every(UCItermsEnum.UCRootFree, self._check_root_free)

        self.up_pre_check_every(UCItermsEnum.UCIsoRight, self._check_iso_right, args=isoPath)

        return self.upCheckFlag

    def _record_hwinfo(self):
        try:
            bus = dbus.SystemBus()
            activation_proxy = bus.get_object("org.freedesktop.activation", "/org/freedesktop/activation")
            activation_bus_interface = dbus.Interface(activation_proxy, dbus_interface="org.freedesktop.activation.interface")
            func_get_encrypted_hardware_info = activation_bus_interface.get_dbus_method("get_encrypted_hardware_info")
            hw_info = None
            if func_get_encrypted_hardware_info != None:
                hw_info, tmp = func_get_encrypted_hardware_info()
                with open(self.CACHE_DIR + "hwinfo", 'w') as fd:
                    fd.write(hw_info)
                fd.close()
            else:
                logging.error("[VM-UP]: method[get_encrypted_hardware_info] not exists.")
                return RMFAILURE
        except Exception as e:
            logging.error("[VM-UP]: Get hw info err[%s].", e)
            return RMFAILURE
        return RMSUCCESS

    def _up_pre_act(self):
        #TODO: 封装
        # 可能安装激活的包，退出原激活服务
        try:
            bus = dbus.SystemBus()
            activation_proxy = bus.get_object("org.freedesktop.activation", "/org/freedesktop/activation")
            activation_bus_interface = dbus.Interface(activation_proxy, dbus_interface="org.freedesktop.activation.interface")
            func_exit = activation_bus_interface.get_dbus_method("exit")
            if func_exit != None:
                func_exit()
            else:
                logging.error("[VM-UP]: method[exit] not exists.")
        except Exception as e:
            logging.error("[VM-UP]: Exit err[%s].", e)
        sleep(1)

        # 获取硬件信息
        ret = self._record_hwinfo()
        if ret != RMSUCCESS:
            sleep(1)
            self._record_hwinfo()

        # 执行升级前处理脚本
        cmd = "bash " + self.SCRIPTS_DIR + self.SCRIPT_BEFORE + " > " + self.LOG_DIR + "upbefore.log"
        get_sys_call(cmd)

        # 记录安全选项关闭列表
        with open(self.CACHE_DIR + self.KYSEC_CFG, 'w+') as fd:
            for ele in self.kysecList:
                fd.write(ele + "\n")
        fd.close()

        # 记录原用户目录, 还原时用
        self.users = get_sys_call("ls -ln /home/ | awk  '{if($3>=1000){printf \"%s \", $NF}}'")
        self.rmStateConf.change_key_value(self.SEC_HOME_INFO, self.KEY_USERS, self.users)

        #备份快捷方式
        self.appClass.backup_desktop()


    def _upFunc(self, func, *param):
        if func == None:
            logging.error("[FUNC]: in func is None.")
            return RMFAILURE
        ret = RMFAILURE
        try:
            logging.info("[FUNC]: func[%s] param[%s].", func.__name__, param)
            if param == None or len(param) == 0:
                ret = func()
            else:
                ret = func(param[0])
        except Exception as e:
            logging.error("[FUNC]: upgrade err[%s]",e)
            self.upStepRet = RMFAILURE
            self.show_up_process(UPIterms[self.upStep], self.upStepRet, self.upProcess, UPHandlerIterms[self.upStep])
            return RMFAILURE

        self.upStepRet = ret
        self.upProcess = self.UP_PROCESS_LIST[self.upStep+1]
        return ret


    def _show_process(self, index = 0, failedCount = 0):
        logging.info("[FUNC]: show info-a [%d %d %d %d]", self.upStep, self.upProcess, self.upStepRet, failedCount)
        if failedCount == 1:
            return
        updateSpeed = 3 # 3秒更新一次

        if self.upStep >= UPItermsEnum.UPItermsMax or self.upStep < 0:
            logging.error("[FUNC]: upstep[%d] is over", self.upStep)

        if self.upProcess < self.UP_PROCESS_LIST[self.upStep+1]:
            self.upProcess = self.upProcess + 1
        
        itermStr = UPIterms[self.upStep]
        
        sug = ""
        if self.upStepRet != RMSUCCESS:
            sug = UPHandlerIterms[self.upStep]
        try:
            self.show_up_process(itermStr, self.upStepRet, self.upProcess, sug)
        except Exception as e:
            logging.error("[FUNC]: sig err occur[%s].", e)

        if self.upProcess >= 59 or self.upProcess < 0:
            logging.info("[FUNC]: process[%d] is over", self.upProcess)
            return
        if self.upStepRet != RMSUCCESS:
            failedCount = 1
            logging.error("[FUNC]: run [%s] failed.", itermStr)
            return

        tmpTimer = threading.Timer(updateSpeed, function=self._show_process, args=[index, failedCount])
        tmpTimer.start()


    def _up_create_init_and_functions(self, action_up, distPath):
        # 升级
        if action_up:
            srcPath = "/usr/share/revisions-manager/data/initramfs-upgrade/"
        # 还原
        else:
            srcPath = "/usr/share/revisions-manager/data/initramfs-back/"
        initFile = srcPath + "init"
        if not os.path.exists(initFile):
            logging.error("[VM-UP]: %s not exists.", initFile)
            return RMFAILURE
        cmd = "cp " + initFile + " " + distPath
        get_sys_call(cmd)

        funcFile = srcPath + "functions"
        if not os.path.exists(funcFile):
            logging.error("[VM-UP]: %s not exists.", funcFile)
            return RMFAILURE
        cmd = "cp " + funcFile + " " + distPath + "scripts/"
        get_sys_call(cmd)
        return RMSUCCESS


    # 创建原系统还原用的initrd
    # TODO:目前是整体文件替换；希望可以仅在文件中添加修改
    def _up_create_orgfs_initrd(self):
        logging.info("[VM-UP]: create orgfs initrd start.")

        ret = self._up_create_init_and_functions(False, "/usr/share/initramfs-tools/")
        if ret == RMFAILURE:
            logging.error("[VM-UP]: _up_create_init_and_functions err ")
            return ret

        cmd = "update-initramfs -u -k " + self.curVersion
        os.system(cmd)

        # 记录生成的initrd md5
        initrdFileName = "initrd.img-"+self.curVersion
        initrdFilePath = os.path.join("/boot", initrdFileName)
        if os.path.exists(initrdFilePath):
            # 备份
            shutil.copyfile(initrdFilePath, self.CACHE_DIR + initrdFileName)

            cmd = "md5sum " + initrdFilePath
            ret = get_sys_call(cmd)

            with open(self.MD5_PATH, "w+") as fd:
                fd.writelines(ret)
                logging.info("[VM-UP]: add md5 info [%s]", ret)
            fd.close()
        else:
            logging.error("[VM-UP]: File[%s] not exists.",initrdFilePath)
            return RMFAILURE

        logging.info("[VM-UP]: create orgfs initrd finished ver[%s].", self.curVersion)
        return RMSUCCESS


    def _check_newfs_safe(self):
        if not os.path.exists(self.NEWFS_DIR):
            return RMSUCCESS
        cmd = "mount | grep \"/fs.new/\" | awk \'{print $3}\' | grep \"^/fs.new/\""
        mountInfoS = get_sys_call(cmd)
        if len(mountInfoS) > 5:
            mountList = mountInfoS.split("\n")
            for ele in mountList:
                if len(ele) < 6:
                    continue
                logging.info("[VM-UP]: [%s] already mounted.", ele)
                cmd = "umount " + ele
                get_sys_call(cmd)

        try:
            # TODO:解除挂载 ok
            shutil.rmtree(self.NEWFS_DIR)
        except OSError as e:
            shutil.rmtree(self.NEWFS_DIR)
            logging.error("[VM-UP]: remove [%s] failed as [%s]", self.NEWFS_DIR, e)
        except Exception as e:
            logging.error("[VM-UP]: remove [%s] failed as [%s]", self.NEWFS_DIR, e)
            return RMFAILURE
        return RMSUCCESS

    # 创建新系统
    # 1-解压文件系统
    # 3-拷贝iso中的源
    # 4-拷贝plymouth脚本
    def _up_create_newfs(self, isoPath):
        logging.info("[VM-UP]: up_create_newfs start")

        if self._check_newfs_safe() == RMFAILURE:
            logging.error("[VM-UP]: check newfs safe err")
            return RMFAILURE

        mountPath, errInfo = mount_iso(isoPath)
        if mountPath is None:
            logging.error("[VM-UP]: Mount [%s] failed.", isoPath)
            return RMFAILURE

        squashfsPath = os.path.join(mountPath, "casper/filesystem.squashfs")
        if not os.path.exists(squashfsPath):
            logging.error("[VM-UP]: iso is broken, please check whether exists filesystem.squashfs[%s]", squashfsPath)
            return RMFAILURE
        else:
            cmd = "unsquashfs -d " + self.NEWFS_DIR + " " + squashfsPath
            logging.info("[VM-UP]: cmd: %s", cmd)
            try:
                shellRet = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                if shellRet.returncode != RMSUCCESS:
                    logging.error("[VM-UP]: cmd[%s] ret[%d] failed as [%s].", cmd, shellRet.returncode, shellRet.stderr.decode())
                    return RMFAILURE
            except Exception as e:
                logging.error("[VM-UP]: cmd[%s] failed as [%s].", cmd, e)

        # iso中的源拷贝并设置
        cdromPath = os.path.join(self.NEWFS_DIR, "cdrom")
        if not os.path.exists(cdromPath):
            os.makedirs(cdromPath, mode=0o755)

        dirList = ["pool", "dists", ".kylin-post-packages", ".kylin-post-actions", ".kylin-post-actions-nochroot", "third-party"]
        for ele in dirList:
            srcPath=os.path.join(mountPath, ele)
            if os.path.exists(srcPath):
                cmd = "cp -r " + srcPath + " " + cdromPath
                get_sys_call(cmd)
            else:
                logging.info("[VM-UP]: [%s] not exists.",srcPath)

        umount_iso(mountPath)

        logging.info("[VM-UP]: create newfs finished")
        return RMSUCCESS

    # 迁移etc
    def _up_move_etc(self):
        logging.info("[VM-UP]: _up_move_etc start")
        moveEtcClass = RMMoveEtc()
        ret = moveEtcClass.move_etc()
        logging.info("[VM-UP]: move etc ret[%d]", ret)
        return ret


    # 创建升级新系统用的initrd
    # 提前安装kylin-revision-manager到新系统
    def _up_inchroot_handle(self):
        logging.info("[VM-UP]: _up_inchroot_handle start")
        new_ker_version=""

        ret = self._up_create_init_and_functions(True, "/fs.new/usr/share/initramfs-tools/")
        if ret == RMFAILURE:
            logging.error("[VM-UP]: install itself err")
            return ret

        newfd=os.open(self.NEWFS_DIR,os.O_RDONLY)
        orgfd=os.open("/",os.O_RDONLY)
        os.chroot(self.NEWFS_DIR)
        os.fchdir(newfd)

        cmd = "linux-version list --paths | linux-version sort --reverse"
        kerVersion = get_sys_call(cmd)
        if len(kerVersion) > 0:
            kerVersionList = str_to_list(kerVersion, "\n")
            KER_VERSION = kerVersionList[0].split(" ")
            new_ker_version = KER_VERSION[0]
        else:
            cmd = "dpkg -l | grep linux-image-[0-9] | awk \'{print $2}\'"
            kerVersion = get_sys_call(cmd)
            kerVersionList = str_to_list(kerVersion, "\n")
            KER_VERSION = kerVersionList[0].split("image-")
            new_ker_version = KER_VERSION[1]

        os.system("mount -t  proc proc /proc")
        os.system("mount -t sysfs sys /sys")
        os.system("mount -t devtmpfs devtmpfs /dev")
        os.system("mount -t devpts devpts /dev/pts")
        try:
            # 安装kylin-revision-manager
            ret = self.appClass.install_revisions_manager()

            if self.osInfo.osNum < self.osInfo.VERSION_OPENKYLIN:
                cmd = "update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth \
                        /usr/share/plymouth/themes/kylin-revision-manager-logo/kylin-revision-manager-logo.plymouth 200"
            else:
                cmd = "update-alternatives --install /usr/share/plymouth/themes/default.plymouth default.plymouth \
                        /usr/share/plymouth/themes/openkylin-revision-manager-logo/openkylin-revision-manager-logo.plymouth 200"
            get_sys_call(cmd)

            get_sys_call("update-initramfs -u")
        except Exception as e:
            logging.error("[VM-UP]: Work in new os err[%s].", e)

        os.system("umount /dev/pts")
        os.system("umount /dev")
        os.system("umount /sys")
        os.system("umount /proc")

        os.fchdir(orgfd)
        os.chroot(".")

        if ret != RMSUCCESS:
            logging.error("[VM-UP]: install itself err")
            return ret

        if os.path.exists("/fs.new/boot/initrd.img-"+new_ker_version):
            os.listdir("/fs.new/boot/")
            cmd = "cp -r /fs.new/boot/*" + new_ker_version + " /boot"
            get_sys_call(cmd)
        else:
            logging.error("[VM-UP]: /fs.new/boot/initrd.img-"+new_ker_version+" not exists")
            return RMFAILURE

        self.newVersion = new_ker_version
        self.rmStateConf.change_key_value(self.SEC_OS_INFO, self.KEY_NEW_KERNEL, new_ker_version)
        logging.info("[VM-UP]: _up_inchroot_handle finished")
        return RMSUCCESS

    # 在 chroot 后执行, 创建升级的GRUB
    def _up_create_newfs_grub(self):
        logging.info("[VM-UP]:_up_create_newfs_grub start.")
        menulist = []
        if self.newVersion is None:
            newBootDir = "/fs.new/boot/"
            kerlist = []
            fileList = os.listdir(newBootDir)
            for ele in fileList:
                if "initrd.img" in ele:
                    if os.path.islink(ele):
                        continue
                    if not ele.endswith("generic"):
                        continue
                    eleSplit = ele.split("-", maxsplit=1)
                    if len(eleSplit) <= 1:
                        continue
                    kerlist.append(eleSplit[1])
            kerlist.sort(reverse=True)
            if len(kerlist) != 0:
                self.newVersion = kerlist[0]

        logging.info("[VM-UP]: new ker version is[%s].", self.newVersion)

        grubClass = RMGrub()
        grubClass.analysis_cfg(grubClass.grubCfgPath)

        # 创建备份
        shutil.copyfile(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg_old-"+self.newVersion))

        grubClass.get_UUID()
        menulist = grubClass.create_upgrade_submenu(self.newVersion)
        if menulist is None or len(menulist) == 0:
            logging.error("[VM-UP]: create menu failed")
            return RMFAILURE

        forcemenulist = grubClass.create_force_org_submenu(self.curVersion)
        if forcemenulist is None or len(forcemenulist) == 0:
            logging.error("[VM-UP]: create force back menu failed")
            return RMFAILURE

        menulist.extend(forcemenulist)

        grubClass.add_subMenu(self.GRUB_FILE_PATH, menulist)
        grubClass.remove_hidden()

        logging.info("[VM-UP]: _up_create_newfs_grub success.")
        return RMSUCCESS
    
    def _up_post_add_back_menu(self):
        logging.info("[VM-UP]:_up_post_add_back_menu start.")
        menulist = []

        oldver = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_KERNEL)
        if oldver is None:
            logging.error("[VM-UP]: Can't get old version.")
            return

        grubClass = RMGrub()
        grubClass.analysis_cfg(grubClass.grubCfgPath)

        # 创建备份
        shutil.copyfile(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg_new"))

        grubClass.get_UUID()

        backmenulist = grubClass.create_fallback_submenu(oldver)
        if backmenulist is None or len(backmenulist) == 0:
            logging.error("[VM-UP]: create force back menu failed")
            return RMFAILURE

        menulist.extend(backmenulist)

        grubClass.add_subMenuEx(self.GRUB_FILE_PATH, menulist)
        grubClass.remove_hidden()

        # 创建备份
        shutil.copyfile(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg_tmp"))

        cmd = "touch /fs.old" + self.ETC_DIR + "need-update-grub"
        get_sys_call(cmd)

        logging.info("[VM-UP]: _up_post_add_back_menu success.")
        return RMSUCCESS


    def _up_compat_app(self):
        logging.info("[VM-UP]: _up_compat_app start.")
        try:
            ret = self.appClass.create_new_deb()
            self.appClass.create_third_party_list()
        except Exception as e:
            logging.error("[VM-UP]: create new app deb err[%s].", e)

        logging.info("[VM-UP]: _up_compat_app finished.")
        return ret

    def up_act(self, isoPath):
        if self.upCheckFlag != RMSUCCESS:
            logging.error("[VM-UP]: pre upgrade check not success")
            return RMFAILURE

        self._up_pre_act()
        tmpTimer = threading.Timer(1, function=self._show_process)
        tmpTimer.start()

        daqStep = "up-act1-failed"
        self.upProcess = 0
        self.daqFailedInfo = ''
        self.upStep = UPItermsEnum.UPCreateOrgFsInitrd
        self.upStepRet = RMSUCCESS
        ret = self._upFunc(self._up_create_orgfs_initrd)
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)
            logging.error("[VM-UP]: action failed and return")
            return ret

        self.upStep = UPItermsEnum.UPCreateNewFs
        ret = self._upFunc(self._up_create_newfs, isoPath)
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)
            logging.error("[VM-UP]: action failed and return")
            return ret

        self.upStep = UPItermsEnum.UPCreateNewDeb
        ret = self._upFunc(self._up_compat_app)
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)   
            logging.error("[VM-UP]: action failed and return")
            return ret

        value = self.rmConf.get_key(self.rmConf.SEC_SWITCH, self.rmConf.KEY_SW_ETC)
        if value.lower() == "true":
            self.upStep = UPItermsEnum.UPMoveEtc
            ret = self._upFunc(self._up_move_etc)
            if ret == RMFAILURE:
                self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)
                logging.error("[VM-UP]: action failed and return")
                return ret

        self.upStep = UPItermsEnum.UPCreateNewFsInitrd
        ret = self._upFunc(self._up_inchroot_handle)
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)
            logging.error("[VM-UP]: action failed and return")
            return ret

        ret = self._check_power(20)
        if ret == RMFAILURE:
            logging.error("[VM-UP]: check power failed")
            self.show_up_process(_("Recheck power failed."), -1, self.UP_PROCESS_LIST[UPItermsEnum.UPItermsMax]-1, _("Power is not enough, please plug in"))
            return ret

        self.upStep = UPItermsEnum.UPCreateNewGrub
        ret = self._upFunc(self._up_create_newfs_grub)
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason(daqStep, UPIterms[self.upStep] + " failed: " + self.daqFailedInfo)
            logging.error("[VM-UP]: action failed and return")
            return ret


        self.show_up_process(UPIterms[self.upStep], 0, self.UP_PROCESS_LIST[UPItermsEnum.UPItermsMax], '')
        self.rmStateConf.change_key_value(self.SEC_OS_INFO, self.KEY_OLD_KERNEL, self.curVersion)
        self.rmStateConf.change_key_value(self.SEC_OS_INFO, self.KEY_OLD_FS_NAME, self.ORGFS_DIR)
        self.versionManagerState = self.intToStr.get(self.VMINIT)
        self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.versionManagerState)
        return RMSUCCESS

    ####################还原####################

    # 与kare约定的处理方案
    # 还原时 在新系统中执行
    def _handle_kare_resolve(self):
        flagPath = "/opt/compatibility_layer/config/ln.flag"
        if not os.path.exists(flagPath):
            logging.info("[VM-BACK]: file %s not exists.", flagPath)
            return
        srcPath = "/var/run/NetworkManager/resolv.conf"
        cmd = "ln -sf " + srcPath + " /fs.old/etc/resolv.conf"
        get_sys_call(cmd)


    # 还原重启后的处理
    # 更新grub initrd
    def _back_post_handle(self):
        logging.info("[VM-BACK]: post handle start")
        self.versionManagerState = self.intToStr.get(self.VMBACKSUCCESS)
        self.rmStateConf.change_key_value(self.SEC_STATE,self.KEY_VM, self.versionManagerState)
        self.appClass.resume_desktop()
        newKerVer = self.rmStateConf.get_key(self.SEC_OS_INFO,self.KEY_NEW_KERNEL)
        if newKerVer != None:
            cmd = "rm -r /boot/*" + newKerVer
            os.system(cmd)

        logging.info("[VM-BACK]: post handle -- update grub")
        self.update_grub()

        logging.info("[VM-BACK]: post handle -- update initrd")
        os.system("update-initramfs -u -k " + self.curVersion)

        self.daqInfo.set_step_and_reason("back-act-success","")
        self.daqInfo.send_all_info()
        logging.info("[VM-BACK]: post handle end")

    # oldfs存在性检查
    def back_check_old_os_exists(self):
        stage = self.rmStateConf.get_key(self.SEC_STATE, self.KEY_CLEARSTAGE)
        if stage != None and stage != "0":
            logging.info("[VM-BACK]: clear stage is [%s].", stage)
            return False
        oldfs = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_FS_NAME)
        if oldfs != None and os.path.exists(oldfs):
            logging.info("[VM-BACK]: old os is [%s].", oldfs)
            return True

        logging.info("[VM-BACK]: %s not exists.", oldfs)
        return False
    
    # oldfs完成性检查
    def _back_check_old_os_complete(self):
        oldfs = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_FS_NAME)
        if not os.path.exists(oldfs):
            logging.error("[VM-BACK]: %s not exists.", oldfs)
            return RMFAILURE

        if not os.path.exists("/usr/share/revisions-manager/"):
            logging.error("[VM-BACK]: revisions-manager not complete.")
            return RMFAILURE
        return RMSUCCESS

    def _back_check_initrd(self):
        oldfs = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_FS_NAME)
        kerVer = self.rmStateConf.get_key(self.SEC_OS_INFO , self.KEY_OLD_KERNEL)
        if kerVer == None:
            logging.error("[VM-BACK]: no old ker version.")
            return RMFAILURE

        md5Str = ""
        if os.path.exists(self.MD5_PATH):
            with open(self.MD5_PATH, "r") as fd:
                md5Str = fd.readlines()
            fd.close()
        if len(md5Str) < 1:
            logging.error("[VM-BACK]: md5 info is err.")
            return RMFAILURE
        valuedist = md5Str[0].split()[0]
        ret = ""
        initrdFilePath = oldfs + self.CACHE_DIR + "initrd.img-" + kerVer
        if os.path.exists(initrdFilePath):
            cmd = "md5sum " + initrdFilePath
            ret = get_sys_call(cmd)
            valueSrc = ret.split()[0]
            if valueSrc != valuedist:
                logging.error("[VM-BACK]: [%s][%s] md5 is diff.", valueSrc, valuedist)
                return RMFAILURE

            shutil.copyfile(initrdFilePath, "/boot/" + "initrd.img-" + kerVer)
            return RMSUCCESS

        logging.error("[VM-BACK]: [%s] is moved.", initrdFilePath)
        return RMFAILURE        

    def back_pre_check(self):
        ret = RMSUCCESS
        ret1 = RMSUCCESS
        ret2 = RMSUCCESS
        sug = ""
        count = 9
        sigSpeed = 5

        try:
            for i in range(count):
                self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOldOs], 0, i * sigSpeed, "")
                sleep(0.3)
            ret1 = self._back_check_old_os_complete()
            ret += ret1
            if ret1 == RMFAILURE:
                self.daqInfo.set_step_and_reason("back-check-failed", FCIterms[FCItermsEnum.FCCheckOldOs] + " failed: ")
                sug = FCHandlerIterms[FCItermsEnum.FCCheckOldOs]
            self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOldOs], ret, 50, sug)
        except Exception as e:
            logging.error("[VM-BACK]: pre Fallback Check err[%s].", e)
            self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOldOs], RMFAILURE, 50, sug)
            self.daqInfo.set_step_and_reason("back-check-failed", FCIterms[FCItermsEnum.FCCheckOldOs] + " failed: ")
            ret = RMFAILURE

        try:
            for i in range(count):
                self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOrgInitrd], 0, i * sigSpeed + 50, "")
                sleep(0.3)
            ret2 = self._back_check_initrd()
            ret += ret2
            if ret2 == RMFAILURE:
                self.daqInfo.set_step_and_reason("back-check-failed", FCIterms[FCItermsEnum.FCCheckOrgInitrd] + " failed: ")
                sug = FCHandlerIterms[FCItermsEnum.FCCheckOrgInitrd]
            self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOrgInitrd], ret, 100, sug)
        except Exception as e:
            logging.error("[VM-BACK]: pre Fallback Check err[%s].", e)
            self.show_back_check_process(FCIterms[FCItermsEnum.FCCheckOrgInitrd], RMFAILURE, 99, sug)
            self.daqInfo.set_step_and_reason("back-check-failed", FCIterms[FCItermsEnum.FCCheckOrgInitrd] + " failed: ")
            ret = RMFAILURE

        return ret
    
    def _back_create_fallback_grub(self):
        kerver = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_KERNEL)
        oldfs = self.rmStateConf.get_key(self.SEC_OS_INFO, self.KEY_OLD_FS_NAME)
        if kerver == None:
            logging.error("[VM-BACK]: no old kerver.")
            return RMFAILURE

        grubClass = RMGrub()
        # /boot不是单独分区的处理   单独封装
        cmd = "lsblk | grep \"/boot\$\" | wc -l"
        ret = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if ret.returncode != 0:
            logging.error("[VM-BACK]: find boot failed [%s].", ret.stderr.decode())
            return RMFAILURE
        retStr = ret.stdout.decode().strip("\n")
        if retStr == '0':
            initrdPath = grubClass.find_initrd(kerver, oldfs + "/boot/")
            vmlinuzPath = grubClass.find_image(kerver, oldfs + "/boot/")
            cmd = "cp " + initrdPath + " /boot/"
            get_sys_call(cmd)
            cmd = "cp " + vmlinuzPath + " /boot/"
            get_sys_call(cmd)

        try:
            # 创建备份
            shutil.copy2(grubClass.grubCfgPath, os.path.join(self.CACHE_DIR, "grub.cfg_new-"+kerver))
            
            grubClass.analysis_cfg(grubClass.grubCfgPath)
            ret = grubClass.get_UUID()
            if ret == RMFAILURE:
                logging.info("[VM-BACK]: get UUID failed")
                return RMFAILURE

            menulist = grubClass.create_fallback_submenu(kerver)
            if menulist is None:
                logging.info("[VM-BACK]: create fallback submenu failed")
                return RMFAILURE
            grubClass.add_subMenu(self.GRUB_FILE_PATH, menulist)
        except Exception as e:
            logging.error("[VM-BACK]: create_fallback_grub failed as [%s].",e)
            return RMFAILURE

        logging.info("[VM-BACK]: create_fallback_grub success.")
        return RMSUCCESS

    def back_act(self):
        sug = ""
        count = 20
        sigSpeed = 5
        # 检测ISO是否符合要求
        for i in range(count):
            self.show_back_process(FPIterms[FPItermsEnum.FPCreateGrub], 0, i * sigSpeed, "")
            sleep(0.3)

        # 创建grub
        ret = self._back_create_fallback_grub()
        if ret == RMFAILURE:
            self.daqInfo.set_step_and_reason("back-act1-failed", FCIterms[FCItermsEnum.FPCreateGrub] + " failed: ")
            sug = FPHandlerIterms[FPItermsEnum.FPCreateGrub]
        self.show_back_process(FPIterms[FPItermsEnum.FPCreateGrub], ret, 100, sug)
        self.versionManagerState = self.intToStr.get(self.VMINIT)
        self.rmStateConf.change_key_value(self.SEC_STATE, self.KEY_VM, self.versionManagerState)

        self._handle_kare_resolve()

        return ret

    # 应用兼容配置
    def get_uncompat_package(self):
        return self.appClass.get_not_compat_package()
    
    # 设置是否迁移第三方应用
    def switch_apps_move(self, enable):
        if enable:
            self.appClass.enable_app_move()
        else:
            self.appClass.disable_app_move()

    def up_getAppList(self):
        self.appClass.find_repack_list()
        return self.appClass.get_app_list()

    def up_setAppList(self, appList):
        return self.appClass.set_app_list(appList)
    
    def up_clearMoveAppList(self):
        return self.appClass.clear_move_app_list()

    def up_set_secList(self, inList):
        self.kysecList = []
        for ele in inList:
            if ele not in self.kysecList:
                self.kysecList.append(ele)
                logging.info("[VM-UP]: add option [%s].", ele)
        return RMSUCCESS

    def up_clear_secList(self):
        self.kysecList = []
        return RMSUCCESS

    def up_get_secList(self):
        return self.kysecList

    def get_state(self):
        return self.versionManagerState
    
    def get_old_os(self):
        return self.ORGFS_DIR
    
    def add_notice(self):
        stateInt = self.strToInt.get(self.versionManagerState)
        if stateInt == self.VMUPFAILED or stateInt == self.VMUPSUCCESS:
            self.versionManagerState = self.intToStr.get(self.VMUP)
        elif stateInt == self.VMBACKFAILED or stateInt == self.VMBACKSUCCESS:
            self.versionManagerState = self.intToStr.get(self.VMBACK)
        else:
            logging.info("[VM-ALL]: VM STATE IS %s.", self.versionManagerState)
            return
        
        self.rmStateConf.change_key_value(self.SEC_STATE, self.KEY_VM, self.versionManagerState)

    def is_overTime(self):
        osName = self.rmStateConf.get_key(self.SEC_OS_INFO,self.KEY_OLD_FS_NAME)
        if not os.path.exists(osName):
            logging.info("[VM] os[%s] not exists.", osName)
            return True

        fmt = "%Y-%m-%d %H:%M:%S"
        uptime = self.rmStateConf.get_key(self.SEC_TIME,self.KEY_UPTIME)
        if uptime is None:
            logging.error("[VM]: there is no uptime")
            return True

        etcConf = RMConf(self.CONF_PATH)
        periodstr = etcConf.get_key(etcConf.SEC_PERIOD,etcConf.KEY_BACKPERIOD)
        periodint = int(periodstr)
        finalTime = datetime.strptime(uptime,fmt) + timedelta(days=periodint)

        tnow = datetime.now()
        if finalTime > tnow:
            return False

        return True
    
    def set_activation_file(self, iPath, lPath):
        if iPath is None or lPath is None:
            logging.error("[VM-UP]: path is none")
            return RMFAILURE
            
        if not os.path.exists(iPath) or not os.path.exists(lPath):
            logging.error("[VM-UP]: file not exist.")
            return RMFAILURE
        
        if not os.path.exists(self.appClass.CACHE_DIR):
            os.makedirs(self.appClass.CACHE_DIR, mode=0o755)
        cmd = "cp -p " + iPath + " " + self.appClass.CACHE_DIR
        get_sys_call(cmd)
        cmd = "cp -p " + lPath + " " + self.appClass.CACHE_DIR
        get_sys_call(cmd)

        logging.info("[V10]: set activation file path success")

    def up_set_default(self):
        self.appClass.set_app_default()
        outList = self.get_default_security_list()
        self.up_set_secList(outList)

    def get_default_security_list(self):
        infoJson = None
        outList = []
        rmRemote = RMRemote()
        displayPath = rmRemote.get_base_display()
        if not os.path.exists(displayPath):
            return outList

        try:
            with open(displayPath,'r') as fd:
                infoJson = json.load(fd)
            fd.close()
            if infoJson == None:
                return outList
            if "page-advance" in infoJson and "option-security" in infoJson["page-advance"] and "checked" in infoJson["page-advance"]["option-security"]:
                ret = infoJson["page-advance"]["option-app"]["checked"]
                tmpDict = infoJson["page-advance"]["option-security"]
                if ret:
                    if "pwquality" in tmpDict and not tmpDict["pwquality"]:
                        outList.append("pwquality")
                    if "userlock" in tmpDict and not tmpDict["userlock"]:
                        outList.append("userlock")               
                    if "exectl" in tmpDict and not tmpDict["exectl"]:
                        outList.append("exectl")
                    if "devctl" in tmpDict and not tmpDict["devctl"]:
                        outList.append("devctl")
        except Exception as e:
            logging.error("[RMRemote]: Get security list err[%s].",e)
        return outList
    
    def update_grub(self):
        logging.info("[RMCOM]: update grub.")
        if os.path.exists("/usr/sbin/update-grub2"):
            os.system("update-grub2")
        elif os.path.exists("/usr/sbin/update-grub"):
            os.system("update-grub")
        else:
            logging.error("Base tools not exists [update-grub]")
            return RMFAILURE
        return RMSUCCESS

    def up_post_update_grub(self):
        if self.update_grub() == RMFAILURE:
            return RMFAILURE
        
        self._up_post_add_back_menu()
        return RMSUCCESS
    
    def up_post_resume_grub(self):
        srcPath = os.path.join(self.CACHE_DIR, "grub.cfg_tmp")
        if os.path.exists(srcPath):
            grubClass = RMGrub()
            cmd = "cp " + os.path.join(self.CACHE_DIR, "grub.cfg_tmp") + " " +  grubClass.grubCfgPath
            get_sys_call(cmd)
            cmd = "sync"
            get_sys_call(cmd)
        else:
            self.up_post_update_grub()

if __name__ == "__main__":
    # testClass = RMVersionManager()
    # testClass._up_create_newfs_grub()
    pass
